@@ -92,13 +92,9 @@ func init() {
 // NewStatCollector returns a new Collector exposing CPU stats.
 func NewStatCollector() (Collector, error) {
 	return &statCollector{
-		cpu: typedDesc{prometheus.NewDesc(
-			prometheus.BuildFQName(namespace, "cpu", "seconds_total"),
-			"Seconds the CPU spent in each mode.",
-			[]string{"cpu", "mode"}, nil,
-		), prometheus.CounterValue},
+		cpu: typedDesc{nodeCPUSecondsDesc, prometheus.CounterValue},
 		temp: typedDesc{prometheus.NewDesc(
-			prometheus.BuildFQName(namespace, "cpu", "temperature_celsius"),
+			prometheus.BuildFQName(namespace, cpuCollectorSubsystem, "temperature_celsius"),
 			"CPU temperature",
 			[]string{"cpu"}, nil,
 		), prometheus.GaugeValue},
